@@ -292,12 +292,14 @@ int main(int argc, char* argv[])
                 saphire_set_array("ARGV", VECTOR_NEW(10));
             }
 
-            sWFd* pipeout = sWFd_new(STDOUT_FILENO);
-            sRFd* pipein = sRFd_new(STDIN_FILENO);
+            sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
+            sRFd* pipein = RFD_NEW(STDIN_FILENO);
             saphire_init_stack_frame();
             int rcode = saphire_load(file_name, pipeout, pipein, STDERR_FILENO);
             saphire_sweep();
             sWFd_flash(pipeout);
+            sRFd_delete(pipein);
+            sWFd_delete(pipeout);
             if(rcode == -1) {
                 fprintf(stderr, "run time err\n");
                 fprintf(stderr, "%s", string_c_str(gErrMsg));
@@ -305,11 +307,22 @@ int main(int argc, char* argv[])
             saphire_final();
             mcurses_final();
 
+            int i;
+            for(i=0; i<vector_size(argv2); i++) {
+                string_delete(vector_item(argv2, i));
+            }
+            vector_delete(argv2);
+
             CHECKML_END();
             return rcode;
         }
     }
 
+    for(i=0; vector_size(argv2); i++) {
+        string_delete(vector_item(argv2, i));
+    }
+    vector_delete(argv2);
+
     /// オプションで指定されているならシェルで実行する ///
     if(send_cmd[0] == 'c') {
         setenv("SAPHIRE_INTERACTIVE", "0", 1);
@@ -323,13 +336,15 @@ int main(int argc, char* argv[])
         */
 
         saphire_init(kATOptC, FALSE, runtime_script, FALSE);
-        sWFd* pipeout = sWFd_new(STDOUT_FILENO);
-        sRFd* pipein = sRFd_new(STDIN_FILENO);
+        sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
+        sRFd* pipein = RFD_NEW(STDIN_FILENO);
 
         int rcode = saphire_shell(send_cmd + 1, "saphire -c", pipeout, pipein, STDERR_FILENO);
         saphire_sweep();
 
         sWFd_flash(pipeout);
+        sWFd_delete(pipeout);
+        sRFd_delete(pipein);
         if(rcode < 0) { 
             fprintf(stderr, "run time err\n");
             fprintf(stderr, "%s", string_c_str(gErrMsg));
@@ -384,7 +399,7 @@ int main(int argc, char* argv[])
         char* prompt = getenv("SAPHIRE_PROMPT");
         if(prompt) {
             string_obj* ret = STRING_NEW("");
-            sRFd* rfd = sRFd_new(0);
+            sRFd* rfd = RFD_NEW(0);
             if(saphire_shell3(ret, prompt, "SAPHIRE_PROMPT", rfd) == -1) {
                 mreset_tty();
                 buf = readline("> ");
@@ -393,6 +408,8 @@ int main(int argc, char* argv[])
                 mreset_tty();
                 buf = readline(string_c_str(ret));
             }
+            sRFd_delete(rfd);
+            string_delete(ret);
         }
         else {
             mreset_tty();
@@ -412,13 +429,15 @@ int main(int argc, char* argv[])
             }
         }
         else if(*buf) {
-            sWFd* pipeout = sWFd_new(STDOUT_FILENO);
-            sRFd* pipein = sRFd_new(STDIN_FILENO);
+            sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
+            sRFd* pipein = RFD_NEW(STDIN_FILENO);
             saphire_init_stack_frame();
             saphire_set_signal();
             int rcode = saphire_shell(buf, "saphire", pipeout, pipein, STDERR_FILENO);
             saphire_sweep();
             sWFd_flash(pipeout);
+            sRFd_delete(pipein);
+            sWFd_delete(pipeout);
             if(rcode == -1) {
                 /// ヒストリに追加 ///
                 add_history(buf);
