@@ -47,10 +47,20 @@ typedef struct
     string_obj* mSource;
 } sStatments;
 
-sStatments* sStatments_new();
-sStatments* sStatments_new2(sStatments* statments);
 void sStatments_delete(sStatments* self);
 
+#if defined(MDEBUG)
+sStatments* sStatments_new_debug(const char* fname, int line, const char* func_name);
+#define STATMENTS_NEW() sStatments_new_debug(__FILE__, __LINE__, __FUNCTION__)
+sStatments* sStatments_new2_debug(sStatments* statments, const char* fname, int line, const char* func_name);
+#define STATMENTS_NEW2(o) sStatments_new2_debug(o, __FILE__, __LINE__, __FUNCTION__)
+#else
+sStatments* sStatments_new();
+#define STATMENTS_NEW() sStatments_new()
+sStatments* sStatments_new2(sStatments* statments);
+#define STATMENTS_NEW2(o) sStatments_new2(o)
+#endif
+
 typedef struct _sClass {
     string_obj* name;      // クラス名
     sStatments* statments; // 命令表
