@@ -229,6 +229,12 @@ void mcurses_init(enum eTerminalKanjiCode code)
 void mcurses_final()
 {
     int i;
+    for(i=0; i<vector_size(gKeyMap); i++) {
+        FREE(vector_item(gKeyMap, i));
+    }
+    vector_delete(gKeyMap);
+
+    FREE(gBuffer);
 
     close(gMCTtyFd);
     
