@@ -255,11 +255,11 @@ sArg* sArg_new2_debug(sArg* arg, const char* fname, int line, const char* func);
 
 sArg* sArg_new(int kind, MANAGED void* body);
 
-#define ARG_NEW(o, o2) sArg_new_debug(o, o2)
+#define ARG_NEW(o, o2) sArg_new(o, o2)
 
 sArg* sArg_new2(sArg* arg);
 
-#define ARG_NEW2(o) sArg_new2_debug(o)
+#define ARG_NEW2(o) sArg_new2(o)
 
 #endif
 
@@ -374,18 +374,6 @@ void sStatment_delete(sStatment* self);
 void sStatment_save(sStatment* self, int fd);
 sStatment* sStatment_load(int fd);
 
-#if defined(MDEBUG)
-sStatments* sStatments_new_debug(const char* fname, int line, const char* func_name);
-#define STATMENTS_NEW() sStatments_new_debug(__FILE__, __LINE__, __FUNCTION__)
-sStatments* sStatments_new2_debug(sStatments* statments, const char* fname, int line, const char* func_name);
-#define STATMENTS_NEW2(o) sStatments_new2_debug(o, __FILE__, __LINE__, __FUNCTION__)
-#else
-sStatments* sStatments_new();
-#define STATMENTS_NEW() sStatments_new()
-sStatment* sStatments_new2(sStatments* statments);
-#define STATMENTS_NEW2(o) sStatments_new2(o)
-#endif
-
 void sStatments_view(sStatments* self);
 
 void sStatments_save(sStatments* self, int fd);
