@@ -353,6 +353,8 @@ static void expand_tilda(string_obj* buf, string_obj* path)
 
     /// 残りを代入
     string_push_back(path, p);
+
+    string_delete(user);
 }
 
 /// 一単語読み込み
@@ -503,7 +505,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
 
                 string_obj* path = STRING_NEW("");
                 expand_tilda(buf, path);
-                vector_add(command->mArgs, sArg_new(0, path));
+                vector_add(command->mArgs, ARG_NEW(0, MANAGED path));
                 string_put(buf, ""); // チルダ展開のパターンはmArgsに追加しない
                 break;
             }
@@ -534,7 +536,8 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
                         char* file = result.gl_pathv[i];
                         if(strcmp(file, ".") != 0 && strcmp(file, "..") != 0)
                         {
-                            vector_add(command->mArgs, sArg_new(0, STRING_NEW(file)));
+                            string_obj* str = STRING_NEW(file);
+                            vector_add(command->mArgs, ARG_NEW(0, MANAGED str));
                         }
                     }
 
@@ -608,6 +611,7 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
     string_obj* str = STRING_NEW("");
     int read_end_of_statment2 = -1;
     if(!read_until_next_close_character('(', ')', p, str, &read_end_of_statment2, FALSE, sname, sline)) {
+        string_delete(str);
         return FALSE;
     }
     string_push_back(not_evaled, string_c_str(str));
@@ -619,10 +623,11 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
 
     if(!parse(string_c_str(str), sname, sline, condition))
     {
-
+        string_delete(str);
+        sStatments_delete(condition);
         return FALSE;
     }
-
+    string_delete(str);
 
     skip_spaces(p, not_evaled);
 
@@ -632,16 +637,25 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
     if(**p == '{') {
         string_obj* str = STRING_NEW("");
         if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
+            string_delete(str);
+            sStatments_delete(condition);
+            sStatments_delete(loop);
             return FALSE;
         }
 
         if(!parse(string_c_str(str), sname, sline, loop))
         {
+            sStatments_delete(condition);
+            sStatments_delete(loop);
+            string_delete(str);
             return FALSE;
         }
+        string_delete(str);
     }
     else {
         err_msg("expected block.", sname, *sline);
+        sStatments_delete(condition);
+        sStatments_delete(loop);
         return FALSE;
     }
 
@@ -675,6 +689,16 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
 
             if(**p != '(') {
                 err_msg("expected (", sname, *sline);
+                int i;
+                for(i=0; i<vector_size(conditions); i++) {
+                    sStatments_delete(vector_item(conditions, i));
+                }
+                vector_delete(conditions);
+                for(i=0; i<vector_size(loops); i++) {
+                    sStatments_delete(vector_item(loops, i));
+                }
+                vector_delete(loops);
+    
                 return FALSE;
             }
 
@@ -682,6 +706,16 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
             string_obj* str = STRING_NEW("");
             int read_end_of_statment = -1;
             if(!read_until_next_close_character('(', ')', p, str, &read_end_of_statment, FALSE, sname, sline)) {
+                string_delete(str);
+                int i;
+                for(i=0; i<vector_size(conditions); i++) {
+                    sStatments_delete(vector_item(conditions, i));
+                }
+                vector_delete(conditions);
+                for(i=0; i<vector_size(loops); i++) {
+                    sStatments_delete(vector_item(loops, i));
+                }
+                vector_delete(loops);
                 return FALSE;
             }
             string_push_back(not_evaled, string_c_str(str));
@@ -689,8 +723,20 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
             sStatments* condition = STATMENTS_NEW();
             if(!parse(string_c_str(str), sname, sline, condition))
             {
+                sStatments_delete(condition);
+                int i;
+                for(i=0; i<vector_size(conditions); i++) {
+                    sStatments_delete(vector_item(conditions, i));
+                }
+                vector_delete(conditions);
+                for(i=0; i<vector_size(loops); i++) {
+                    sStatments_delete(vector_item(loops, i));
+                }
+                vector_delete(loops);
+                string_delete(str);
                 return FALSE;
             }
+            string_delete(str);
 
             vector_add(conditions, condition);
 
@@ -702,16 +748,49 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
             if(**p == '{') {
                 string_obj* str = STRING_NEW("");
                 if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
+                    string_delete(str);
+                    sStatments_delete(loop);
+                    int i;
+                    for(i=0; i<vector_size(conditions); i++) {
+                        sStatments_delete(vector_item(conditions, i));
+                    }
+                    vector_delete(conditions);
+                    for(i=0; i<vector_size(loops); i++) {
+                        sStatments_delete(vector_item(loops, i));
+                    }
+                    vector_delete(loops);
                     return FALSE;
                 }
 
                 if(!parse(string_c_str(str), sname, sline, loop))
                 {
+                    sStatments_delete(loop);
+                    string_delete(str);
+                    int i;
+                    for(i=0; i<vector_size(conditions); i++) {
+                        sStatments_delete(vector_item(conditions, i));
+                    }
+                    vector_delete(conditions);
+                    for(i=0; i<vector_size(loops); i++) {
+                        sStatments_delete(vector_item(loops, i));
+                    }
+                    vector_delete(loops);
                     return FALSE;
                 }
+                string_delete(str);
             }
             else {
                 err_msg("expected block.", sname, *sline);
+                sStatments_delete(loop);
+                int i;
+                for(i=0; i<vector_size(conditions); i++) {
+                    sStatments_delete(vector_item(conditions, i));
+                }
+                vector_delete(conditions);
+                for(i=0; i<vector_size(loops); i++) {
+                    sStatments_delete(vector_item(loops, i));
+                }
+                vector_delete(loops);
                 return FALSE;
             }
 
@@ -741,16 +820,47 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
         if(**p == '{') {
             string_obj* str = STRING_NEW("");
             if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
+                string_delete(str);
+                sStatments_delete(loop);
+                int i;
+                for(i=0; i<vector_size(conditions); i++) {
+                    sStatments_delete(vector_item(conditions, i));
+                }
+                vector_delete(conditions);
+                for(i=0; i<vector_size(loops); i++) {
+                    sStatments_delete(vector_item(loops, i));
+                }
+                vector_delete(loops);
                 return FALSE;
             }
 
             if(!parse(string_c_str(str), sname, sline, loop))
             {
+                string_delete(str);
+                sStatments_delete(loop);
+                int i;
+                for(i=0; i<vector_size(conditions); i++) {
+                    sStatments_delete(vector_item(conditions, i));
+                }
+                vector_delete(conditions);
+                for(i=0; i<vector_size(loops); i++) {
+                    sStatments_delete(vector_item(loops, i));
+                }
                 return FALSE;
             }
+            string_delete(str);
         }
         else {
             err_msg("expected block.", sname, *sline);
+            int i;
+            for(i=0; i<vector_size(conditions); i++) {
+                sStatments_delete(vector_item(conditions, i));
+            }
+            vector_delete(conditions);
+            for(i=0; i<vector_size(loops); i++) {
+                sStatments_delete(vector_item(loops, i));
+            }
+            sStatments_delete(loop);
             return FALSE;
         }
 
@@ -760,7 +870,7 @@ static BOOL read_command_parse_if_statment(char** p, sCommand* command, sStatmen
     }
     
     command->mKind = kIf;
-    command->mExtra = sIf_new(conditions, loops);
+    command->mExtra = sIf_new(MANAGED conditions, MANAGED loops);
 
     return TRUE;
 }
@@ -779,6 +889,7 @@ static BOOL read_command_parse_while_statment(char** p, sCommand* command, sStat
     string_obj* str = STRING_NEW("");
     int read_end_of_statment2 = -1;
     if(!read_until_next_close_character('(', ')', p, str, &read_end_of_statment2, FALSE, sname, sline)) {
+        string_delete(str);
         return FALSE;
     }
     string_push_back(not_evaled, string_c_str(str));
@@ -789,8 +900,11 @@ static BOOL read_command_parse_while_statment(char** p, sCommand* command, sStat
     sStatments* conditions = STATMENTS_NEW();
     if(!parse(string_c_str(str), sname, sline, conditions))
     {
+        sStatments_delete(conditions);
+        string_delete(str);
         return FALSE;
     }
+    string_delete(str);
 
     while(**p) {
         if(**p == '\n') {
@@ -813,23 +927,33 @@ static BOOL read_command_parse_while_statment(char** p, sCommand* command, sStat
     if(**p == '{') {
         string_obj* str = STRING_NEW("");
         if(!read_block(p, str, NULL, not_evaled, FALSE, sname, sline)) {
+            sStatments_delete(conditions);
+            sStatments_delete(loops);
+            string_delete(str);
             return FALSE;
         }
 
         if(!parse(string_c_str(str), sname, sline, loops))
         {
+            string_delete(str);
+            sStatments_delete(conditions);
+            sStatments_delete(loops);
             return FALSE;
         }
+
+        string_delete(str);
     }
     else {
         err_msg("expected block.", sname, *sline);
+        sStatments_delete(conditions);
+        sStatments_delete(loops);
         return FALSE;
     }
 
     command->mKind = kWhile;
         // これが無いとmNotEvaledの場合（変数がある場合）
         // 正しくcommandが消されない
-    command->mExtra = sWhile_new(conditions, loops);
+    command->mExtra = sWhile_new(MANAGED conditions, MANAGED loops);
 
     return TRUE;
 }
@@ -840,7 +964,9 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
     while(**p) {
         string_obj* buf = STRING_NEW("");
 
-        if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, command, sname, sline)) {
+        if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, command, sname, sline)) 
+        {
+            string_delete(buf);
             return FALSE;
         }
 
@@ -848,7 +974,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
         if(strcmp(string_c_str(buf), "") != 0) {
             if(vector_size(command->mArgs) == 0) {
                 if(strcmp(string_c_str(buf), "if") == 0) {
-                    vector_add(command->mArgs, sArg_new(0, buf));
+                    vector_add(command->mArgs, ARG_NEW(0, MANAGED buf));
                     string_push_back(statment->mTitle, string_c_str(buf));
                     string_push_back(statment->mTitle, " ");
 
@@ -858,7 +984,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                     }
                 }
                 else if(strcmp(string_c_str(buf), "while") == 0) {
-                    vector_add(command->mArgs, sArg_new(0, buf));
+                    vector_add(command->mArgs, ARG_NEW(0, MANAGED buf));
                     string_push_back(statment->mTitle, string_c_str(buf));
                     string_push_back(statment->mTitle, " ");
 
@@ -873,13 +999,13 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         statment->mRCodeReverse = TRUE;
                     }
 
-                    vector_add(command->mArgs, sArg_new(0, buf));
+                    vector_add(command->mArgs, ARG_NEW(0, MANAGED buf));
                     string_push_back(statment->mTitle, string_c_str(buf));
                     string_push_back(statment->mTitle, " ");
                 }
             }
             else {
-                vector_add(command->mArgs, sArg_new(0, buf));
+                vector_add(command->mArgs, ARG_NEW(0, MANAGED buf));
                 string_push_back(statment->mTitle, string_c_str(buf));
                 string_push_back(statment->mTitle, " ");
             }
@@ -889,7 +1015,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
             if(**p == '\'') {
                 string_push_back(not_evaled, "''");
                 (*p)+=2;
-                vector_add(command->mArgs, sArg_new(0, buf));
+                vector_add(command->mArgs, ARG_NEW(0, MANAGED buf));
                 string_push_back(statment->mTitle, "'");
                 string_push_back(statment->mTitle, " ");
 
@@ -898,14 +1024,17 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
             else if(**p == '"') {
                 string_push_back(not_evaled, "\"\"");
                 (*p)+=2;
-                vector_add(command->mArgs, sArg_new(0, buf));
+                vector_add(command->mArgs, ARG_NEW(0, MANAGED buf));
                 string_push_back(statment->mTitle, "\"\"");
                 string_push_back(statment->mTitle, " ");
 
                 continue;
             }
+            else {
+                string_delete(buf);
+            }
         }
-        
+
         /// クィックコマンド展開 ///
         if((**p == '@' && *(*p+1) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) >= 'a' && *(*p+2) <= 'z' && *(*p+3) == '(')) 
             {
@@ -951,6 +1080,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
             string_obj* str = STRING_NEW("");
             int read_end_of_statment = -1;
             if(!read_until_next_close_character('(', ')', p, str, &read_end_of_statment, FALSE, sname, sline)) {
+                string_delete(str);
                 return FALSE;
             }
             string_push_back(not_evaled, string_c_str(str));
@@ -962,14 +1092,18 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
             sStatments* statments = STATMENTS_NEW();
             if(!parse(string_c_str(str), sname, sline, statments))
             {
+                string_delete(str);
+                sStatments_delete(statments);
                 return FALSE;
             }
 
+            string_delete(str);
+
             if(double_) {
-                vector_add(command->mArgs, sArg_new(4, sAtCommand_new(statments, lf)));
+                vector_add(command->mArgs, ARG_NEW(4, MANAGED sAtCommand_new(MANAGED statments, lf)));
             }
             else {
-                vector_add(command->mArgs, sArg_new(2, statments));
+                vector_add(command->mArgs, ARG_NEW(2, MANAGED statments));
             }
         }
         else if(**p == '>') {
@@ -980,6 +1114,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 string_obj* cmd = STRING_NEW("");
                 if(!read_until_next_close_character('(', ')', p, cmd, read_end_of_statment, FALSE, sname, sline)) {
+                    string_delete(cmd);
                     return FALSE;
                 }
                 string_push_back(not_evaled, string_c_str(cmd));
@@ -989,10 +1124,14 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 sStatments* statments = STATMENTS_NEW();
                 if(!parse(string_c_str(cmd), sname, sline, statments)) {
+                    sStatments_delete(statments);
+                    string_delete(cmd);
                     return FALSE;
                 }
 
-                vector_add(command->mArgs, sArg_new(8, statments));
+                string_delete(cmd);
+
+                vector_add(command->mArgs, ARG_NEW(8, MANAGED statments));
             }
             /// リダイレクト
             else if(*(*p+1) == '>') {
@@ -1003,7 +1142,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 skip_spaces(p, not_evaled);
 
-                vector_add(command->mArgs, sArg_new(9, sRedirect_new(type, fd)));
+                vector_add(command->mArgs, ARG_NEW(9, MANAGED sRedirect_new(type, fd)));
             }
             /// リダイレクト
             else {
@@ -1014,7 +1153,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 skip_spaces(p, not_evaled);
 
-                vector_add(command->mArgs, sArg_new(9, sRedirect_new(type, fd)));
+                vector_add(command->mArgs, ARG_NEW(9, MANAGED sRedirect_new(type, fd)));
             }
         }
         else if(**p == '<') {
@@ -1053,6 +1192,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         skip_spaces(p, not_evaled);
                         if(**p != '\n') {
                             err_msg("invalid here document name", sname, *sline);
+                            string_delete(name);
                             return FALSE;
                         }
                         string_push_back2(not_evaled, **p);
@@ -1073,6 +1213,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         skip_spaces(p, not_evaled);
                         if(**p != '\n') {
                             err_msg("invalid here document name", sname, *sline);
+                            string_delete(name);
                             return FALSE;
                         }
                         string_push_back2(not_evaled, **p);
@@ -1087,6 +1228,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         skip_spaces(p, not_evaled);
                         if(**p != '\n') {
                             err_msg("invalid here document name", sname, *sline);
+                            string_delete(name);
                             return FALSE;
                         }
                         string_push_back2(not_evaled, **p);
@@ -1095,17 +1237,20 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                     }
                     else {
                         err_msg("invalid here document name", sname, *sline);
+                        string_delete(name);
                         return FALSE;
                     }
                 }
 
                 if(dquote || squote) {
                     err_msg("invalid here document name", sname, *sline);
+                    string_delete(name);
                     return FALSE;
                 }
                 
                 if(string_c_str(name)[0] == 0) {
                     err_msg("invalid here document name", sname, *sline);
+                    string_delete(name);
                     return FALSE;
                 }
 
@@ -1134,6 +1279,9 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         }
 
                         if(!read_until_next_close_character('(', ')', p, not_evaled, read_end_of_statment, TRUE, sname, sline)) {
+                            string_delete(name);
+                            string_delete(line);
+                            string_delete(text);
                             return FALSE;
                         }
 
@@ -1151,7 +1299,10 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                     }
                 }
 
-                vector_add(command->mArgs, sArg_new(7, text));
+                vector_add(command->mArgs, ARG_NEW(7, MANAGED text));
+
+                string_delete(line);
+                string_delete(name);
             }
             /// プロセス置換 ///
             else if(*(*p+1) == '(') {
@@ -1160,6 +1311,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 string_obj* cmd = STRING_NEW("");
                 if(!read_until_next_close_character('(', ')', p, cmd, read_end_of_statment, FALSE, sname, sline)) {
+                    string_delete(cmd);
                     return FALSE;
                 }
                 string_push_back(not_evaled, string_c_str(cmd));
@@ -1169,10 +1321,14 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 sStatments* statments = STATMENTS_NEW();
                 if(!parse(string_c_str(cmd), sname, sline, statments)) {
+                    string_delete(cmd);
+                    sStatments_delete(statments);
                     return FALSE;
                 }
 
-                vector_add(command->mArgs, sArg_new(6, statments));
+                string_delete(cmd);
+
+                vector_add(command->mArgs, ARG_NEW(6, MANAGED statments));
             }
             /// リダイレクト
             else {
@@ -1183,7 +1339,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 skip_spaces(p, not_evaled);
                 
-                vector_add(command->mArgs, sArg_new(9, sRedirect_new(type, fd)));
+                vector_add(command->mArgs, ARG_NEW(9, MANAGED sRedirect_new(type, fd)));
             }
         }
         /// リダイレクト ///
@@ -1201,7 +1357,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 skip_spaces(p, not_evaled);
 
-                vector_add(command->mArgs, sArg_new(9, sRedirect_new(type, fd)));
+                vector_add(command->mArgs, ARG_NEW(9, MANAGED sRedirect_new(type, fd)));
 
             }
             else if(*(*p+1) == '2' && *(*p+2) == '>') {
@@ -1212,7 +1368,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 skip_spaces(p, not_evaled);
 
-                vector_add(command->mArgs, sArg_new(9, sRedirect_new(type, fd)));
+                vector_add(command->mArgs, ARG_NEW(9, MANAGED sRedirect_new(type, fd)));
 
             }
             else if(*(*p+1) == '>') {
@@ -1224,7 +1380,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
                 skip_spaces(p, not_evaled);
 
-                vector_add(command->mArgs, sArg_new(9, sRedirect_new(type, fd)));
+                vector_add(command->mArgs, ARG_NEW(9, MANAGED sRedirect_new(type, fd)));
 
             }
             else {
@@ -1244,6 +1400,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                 string_obj* str = STRING_NEW("");
                 int read_end_of_statment = -1;
                 if(!read_until_next_close_character('(', ')', p, str, &read_end_of_statment, FALSE, sname, sline)) {
+                    string_delete(str);
                     return FALSE;
                 }
                 string_push_back(not_evaled, string_c_str(str));
@@ -1255,12 +1412,16 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                 sStatments* statments = STATMENTS_NEW();
                 if(!parse(string_c_str(str), sname, sline, statments))
                 {
+                    string_delete(str);
+                    sStatments_delete(statments);
                     return FALSE;
                 }
 
+                string_delete(str);
+
                 command->mKind= kSubshell;
-                vector_add(command->mArgs, sArg_new(0, STRING_NEW("subshell")));
-                command->mExtra = sSubshell_new(statments);
+                vector_add(command->mArgs, ARG_NEW(0, MANAGED STRING_NEW("subshell")));
+                command->mExtra = sSubshell_new(MANAGED statments);
             }
             else {
                 char buf[128];
@@ -1274,12 +1435,25 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
             string_obj* str = STRING_NEW("");
             vector_obj* args = VECTOR_NEW(10);
             if(!read_block(p, str, args, not_evaled, FALSE, sname, sline)) {
+                int i;
+                for(i=0; i<vector_size(args); i++) {
+                    string_delete(vector_item(args, i));
+                }
+                vector_delete(args);
+                string_delete(str);
                 return FALSE;
             }
 
             sStatments* block = STATMENTS_NEW();
             if(!parse(string_c_str(str), sname, sline, block))
             {
+                sStatments_delete(block);
+                int i;
+                for(i=0; i<vector_size(args); i++) {
+                    string_delete(vector_item(args, i));
+                }
+                vector_delete(args);
+                string_delete(str);
                 return FALSE;
             }
 
@@ -1292,12 +1466,14 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                 }
             }
 
-            vector_add(command->mArgs, sArg_new(5, sBlock_new(block, args, count-1)));
+            vector_add(command->mArgs, ARG_NEW(5, MANAGED sBlock_new(MANAGED block, MANAGED args, count-1)));
 
             string_push_back(statment->mTitle, string_c_str(str));
             string_push_back(statment->mTitle, " ");
 
             skip_spaces(p, not_evaled);
+
+            string_delete(str);
         }
         else {
             break;
@@ -1357,6 +1533,7 @@ static BOOL read_statment(char**p, sStatment* statment, string_obj* not_evaled,
         sCommand* command = COMMAND_NEW();
         if(!read_command(p, command, statment, not_evaled, read_end_of_statment, sname, sline)) 
         {
+            sCommand_delete(command);
             return FALSE;
         }
 
@@ -1371,10 +1548,14 @@ static BOOL read_statment(char**p, sStatment* statment, string_obj* not_evaled,
             }
             else {
                 err_msg("invalid command name", sname, *sline);
+                sCommand_delete(command);
                 return FALSE;
             }
 
-            vector_add(statment->mCommands, command);
+            vector_add(statment->mCommands, MANAGED command);
+        }
+        else {
+            sCommand_delete(command);
         }
 
         if(**p == '|' && *(*p+1) != '>' && (*(*p+1) < '0' || *(*p+1) > '9') && *(*p+1) != '|')
@@ -1411,6 +1592,8 @@ BOOL parse(char* cmdline, char* sname, int* sline, sStatments* statments)
 
         if(!read_statment(&p, statment, not_evaled, &read_end_of_statment, sname, sline))
         {
+            sStatment_delete(statment);
+            string_delete(not_evaled);
             return FALSE;
         }
 
@@ -1515,17 +1698,25 @@ BOOL parse(char* cmdline, char* sname, int* sline, sStatments* statments)
             char buf[128];
             snprintf(buf, 128, "unexpected token3 -->%c\n", *p);
             err_msg(buf, sname, *sline);
+            sStatment_delete(statment);
+            string_delete(not_evaled);
             return FALSE;
         }
 
         /// コマンド展開があったなら
         if(read_end_of_statment != -1) {
-            statment->mNotEvaled = not_evaled;
+            statment->mNotEvaled = MANAGED not_evaled;
+        }
+        else {
+            string_delete(not_evaled);
         }
 
         // コマンドがあるか、コマンド展開があったら複文に文を追加
         if(vector_size(statment->mCommands) > 0 || statment->mNotEvaled != NULL) {
-            vector_add(statments->mStatments, statment);
+            vector_add(statments->mStatments, MANAGED statment);
+        }
+        else {
+            sStatment_delete(statment);
         }
     }
 
@@ -1627,6 +1818,7 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
                 int read_end_of_statment = -1;
                 if(!read_until_next_close_character('(', ')', &p, str, &read_end_of_statment, FALSE, sname, sline)) 
                 {
+                    string_delete(str);
                     return FALSE;
                 }
 
@@ -1637,7 +1829,10 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
                 char* fname = MALLOC(strlen(sname) + 256);
                 snprintf(fname, strlen(sname) + 256, "%s %d: expand command", sname, *sline);
                 int rcode = saphire_shell5(ret, string_c_str(str), fname, rfd, object);
+                FREE(fname);
+                string_delete(str);
                 if(rcode < 0) {
+                    string_delete(ret);
                     return FALSE;
                 }
 
@@ -1645,13 +1840,18 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
                     string_obj* quoted = STRING_NEW("");
                     if(!expand_env_get_quoted_str(string_c_str(ret), quoted, sname, sline))
                     {
+                        string_delete(ret);
+                        string_delete(quoted);
                         return FALSE;
                     }
                     string_push_back(out, string_c_str(quoted));
+                    string_delete(quoted);
                 }
                 else {
                     string_push_back(out, string_c_str(ret));
                 }
+
+                string_delete(ret);
             }
             else {
                 string_push_back2(out, *p++);
@@ -1670,8 +1870,10 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
         else if(*p == '{') {
             string_obj* dummy = STRING_NEW("");
             if(!read_block(&p, dummy, NULL, out, FALSE, sname, sline)) {
+                string_delete(dummy);
                 return FALSE;
             }
+            string_delete(dummy);
         }
         /// サブシェル、条件式は飛ばす
         else if(*p == '(') {
@@ -1752,6 +1954,7 @@ BOOL saphire_compile(char* fname, char* out_fname)
 
     int f = open(fname, O_RDONLY);
     if(f < 0) {
+        string_delete(str);
         return FALSE;
     }
 
@@ -1763,6 +1966,7 @@ BOOL saphire_compile(char* fname, char* out_fname)
         }
         if(size < 0) {
             close(f);
+            string_delete(str);
             return FALSE;
         }
 
@@ -1773,22 +1977,26 @@ BOOL saphire_compile(char* fname, char* out_fname)
     close(f);
     
     /// パース ///
-    sStatments* statments = STATMENTS_NEW();
-
     int sline = 1;
 
     string_obj* str2 = STRING_NEW("");
     if(!delete_comments(string_c_str(str), str2, " ", fname, &sline)) {
-
+        string_delete(str);
+        string_delete(str2);
         return FALSE;
     }
+    string_delete(str);
 
     sline = 1;
 
+    sStatments* statments = STATMENTS_NEW();
     if(!parse(string_c_str(str2), fname, &sline, statments)) {
+        sStatments_delete(statments);
+        string_delete(str2);
         return FALSE;
     }
-    
+
+    string_delete(str2);
 
     /// コンパイル結果を保存 ///
     int fd = open(out_fname, O_WRONLY|O_TRUNC|O_CREAT, 0644);
@@ -1801,6 +2009,8 @@ BOOL saphire_compile(char* fname, char* out_fname)
 
     (void)close(fd);
 
+    sStatments_delete(statments);
+
     return TRUE;
 }
 
@@ -1840,5 +2050,12 @@ int saphire_load_obj(char* fname, sWFd* pipeout, sRFd* pipein, int pipeerr, vect
     /// 解放 ///
     close(fd);
 
+    int i;
+    for(i=0; i<vector_size(envs); i++) {
+        string_delete(vector_item(envs, i));
+    }
+    vector_delete(envs);
+    sStatments_delete(statments);
+
     return rcode;
 }
