@@ -21,15 +21,23 @@ typedef struct {
 // ֐`
 //////////////////////////////////////////////////////////////////////
 // 
+#if defined(MDEBUG)
+vector_obj* vector_new_debug(int first_size, const char* fname, int line, const char* func_name);
+
+#define VECTOR_NEW(size) vector_new_debug(size, __FILE__, __LINE__, __FUNCTION__)
+#else
 vector_obj* vector_new(int first_size);
+
 #define VECTOR_NEW(size) vector_new(size)
 
-void vector_delete(vector_obj* self);                           // 
+#endif
 
 vector_obj* vector_clone(vector_obj* v);                        // N[
 
 #define vector_item(o, i) ((vector_obj*)o)->mTable[i]
 
+void vector_delete(vector_obj* self);
+
 //void* vector_item(vector_obj* self, int index);                 // ACeԂ
 /*
 //////////////////////////////////////////////////////////////////////
